<?php

namespace shqawe\SST\XF\Pub\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;

class Forum extends XFCP_Forum{

    public function actionForum(ParameterBag $params)
	{

        $this->assertNotEmbeddedImageRequest();

		$forum = $this->assertViewableForum($params->node_id ?: $params->node_name, $this->getForumViewExtraWith());
		$forumTypeHandler = $forum->TypeHandler;

		$overrideReply = $forumTypeHandler->overrideDisplay($forum, $this);
		if ($overrideReply)
		{
			return $overrideReply;
		}

		if ($this->responseType == 'rss')
		{
			$this->assertNotEmbeddedImageRequest();

			return $this->getForumRss($forum);
		}

		$filters = $this->getForumFilterInput($forum);

		$page = $this->filterPage($params->page);
		$perPage = $forumTypeHandler->getThreadsPerPage($forum);

		$this->assertCanonicalUrl($this->buildLink('forums', $forum, ['page' => $page]));

		$threadRepo = $this->getThreadRepo();

		$threadList = $threadRepo->findThreadsForForumView(
			$forum, [
			'allowOwnPending' => $this->hasContentPendingApproval()
		]
		);
		$this->applyForumFilters($forum, $threadList, $filters);
		$forum->TypeHandler->adjustForumThreadListFinder($forum, $threadList, $page, $this->request);

		if ($page == 1)
		{
			$stickyThreadList = clone $threadList;
			$stickyThreadLev2List = clone $threadList;

			/** @var \XF\Entity\Thread[]|\XF\Mvc\Entity\AbstractCollection $stickyThreads */
			$stickyThreads = $stickyThreadList->where('sticky', 1)->fetch();
			$stickyThreadLev2 = $stickyThreadLev2List->where('sec_sticky', 1)->fetch();
		}
		else
		{
			$stickyThreads = null;
			$stickyThreadLev2 = null;
		}



		$this->applyDateLimitFilters($forum, $threadList, $filters);

		$threadList->where('sticky', 0)
        ->where('sec_sticky', 0)
			->limitByPage($page, $perPage);

		/** @var \XF\Entity\Thread[]|\XF\Mvc\Entity\AbstractCollection $threads */
		$threads = $threadList->fetch();
		$totalThreads = $threadList->total();

		$this->assertValidPage($page, $perPage, $totalThreads, 'forums', $forum->Node);

		$nodeRepo = $this->getNodeRepo();
		$nodes = $nodeRepo->getNodeList($forum->Node);
		$nodeTree = count($nodes) ? $nodeRepo->createNodeTree($nodes, $forum->node_id) : null;
		$nodeExtras = $nodeTree ? $nodeRepo->getNodeListExtras($nodeTree) : null;

		$allThreads = $threads;
		if ($stickyThreads)
		{
			$allThreads->merge($stickyThreads);
		}

        if ($stickyThreadLev2)
		{
			$allThreads->merge($stickyThreadLev2);
		}

		$forumTypeHandler->fetchExtraContentForThreadsFullView($forum, $allThreads);

		$canInlineMod = false;
		if ($stickyThreads)
		{
			foreach ($stickyThreads AS $thread)
			{
				if ($thread->canUseInlineModeration())
				{
					$canInlineMod = true;
					break;
				}
			}
		}

        if ($stickyThreadLev2)
		{
			foreach ($stickyThreadLev2 AS $thread)
			{
				if ($thread->canUseInlineModeration())
				{
					$canInlineMod = true;
					break;
				}
			}
		}

		foreach ($threads AS $thread)
		{
			if ($thread->canUseInlineModeration())
			{
				$canInlineMod = true;
				break;
			}
		}

		// if the forum is unread and perhaps it shouldn't be, see if we can mark it AS read
		if (\XF::visitor()->user_id
			&& $page == 1
			&& !$filters
			&& $forum->isUnread()
		)
		{
			$hasNew = false;
			foreach ($threads AS $thread)
			{
				if ($thread->isUnread() && !$thread->isIgnored())
				{
					$hasNew = true;
					break;
				}
			}

			if (!$hasNew)
			{
				$this->getForumRepo()->markForumReadIfPossible($forum);
			}
		}

		if (!empty($filters['starter_id']))
		{
			$starterFilter = $this->em()->find('XF:User', $filters['starter_id']);
		}
		else
		{
			$starterFilter = null;
		}

		if (!empty($filters['thread_type']) && $forum->TypeHandler->isThreadTypeAllowed($filters['thread_type'], $forum))
		{
			$threadTypeFilter = $this->app->threadType($filters['thread_type'], false);
		}
		else
		{
			$threadTypeFilter = null;
		}

		$isDateLimited = (empty($filters['no_date_limit']) && (!empty($filters['last_days']) || $forum->list_date_limit_days));
		$threadEndOffset = ($page - 1) * $perPage + count($threads);
		$showDateLimitDisabler = ($isDateLimited && $threadEndOffset >= $totalThreads);

		$canonicalFilters = [];
		if ($page > 1 && $forum->list_date_limit_days && empty($filters['order']))
		{
			$cutOff = \XF::$time - ($forum->list_date_limit_days * 86400);
			$lastThread = $threads->last();
			if (
				$showDateLimitDisabler
				|| ($lastThread && $lastThread->last_post_date <= $cutOff)
			)
			{
				// we have removed the date limit and the last thread here is only shown because of that
				$canonicalFilters['no_date_limit'] = 1;
			}
		}

		$isPreRegQuickThread = $forum->canCreateThreadPreReg();
		if ($forum->canCreateThread() || $isPreRegQuickThread)
		{
			$quickThreadPrefixes = \XF::asPreRegActionUserIfNeeded(
				$isPreRegQuickThread,
				function() use ($forum) { return $forum->getUsablePrefixes(); }
			);
		}
		else
		{
			$quickThreadPrefixes = [];
		}

		$overrideContext = [
			'page' => $page,
			'filters' => $filters
		];

		$viewParams = [
			'forum' => $forum,

			'canInlineMod' => $canInlineMod,

			'nodeTree' => $nodeTree,
			'nodeExtras' => $nodeExtras,

			'stickyThreads' => $stickyThreads,
            'stickyThreadLev2' => $stickyThreadLev2, 
			'threads' => $threads,

			'page' => $page,
			'perPage' => $perPage,
			'total' => $totalThreads,

			'filters' => $filters,
			'canonicalFilters' => $canonicalFilters,
			'starterFilter' => $starterFilter,
			'threadTypeFilter' => $threadTypeFilter,
			'showDateLimitDisabler' => $showDateLimitDisabler,

			'sortInfo' => $this->getEffectiveSortInfo($forum, $filters),

			'pendingApproval' => $this->filter('pending_approval', 'bool'),

			'templateOverrides' => $forumTypeHandler->getForumViewTemplateOverrides($forum, $overrideContext),

			'quickThreadPrefixes' => $quickThreadPrefixes
		];

		list($viewClass, $viewTemplate) = $forumTypeHandler->getForumViewAndTemplate($forum);
		$viewParams = $forumTypeHandler->adjustForumViewParams($forum, $viewParams, $this->request);

		return $this->view($viewClass, $viewTemplate, $viewParams);
		
	}



}


